/*
 * Password Management Servlets (PWM)
 * http://www.pwm-project.org
 *
 * Copyright (c) 2006-2009 Novell, Inc.
 * Copyright (c) 2009-2018 The PWM Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package password.pwm.config.value;

import password.pwm.PwmConstants;
import password.pwm.config.PwmSetting;
import password.pwm.config.StoredValue;
import password.pwm.error.PwmUnrecoverableException;
import password.pwm.i18n.Display;
import password.pwm.util.java.JsonUtil;
import password.pwm.util.java.XmlElement;
import password.pwm.util.java.XmlFactory;
import password.pwm.util.secure.PwmSecurityKey;

import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

public class BooleanValue implements StoredValue
{
    boolean value;

    public BooleanValue( final boolean value )
    {
        this.value = value;
    }


    public static StoredValueFactory factory( )
    {
        return new StoredValueFactory()
        {
            public BooleanValue fromJson( final String value )
            {
                return new BooleanValue( JsonUtil.deserialize( value, Boolean.class ) );
            }

            public BooleanValue fromXmlElement( final PwmSetting pwmSetting, final XmlElement settingElement, final PwmSecurityKey input )
            {
                final XmlElement valueElement = settingElement.getChild( "value" );
                final String value = valueElement.getText();
                return new BooleanValue( Boolean.valueOf( value ) );
            }

        };
    }

    public List<String> validateValue( final PwmSetting pwmSetting )
    {
        return Collections.emptyList();
    }

    @Override
    public List<XmlElement> toXmlValues( final String valueElementName, final PwmSecurityKey pwmSecurityKey  )
    {
        final XmlElement valueElement = XmlFactory.getFactory().newElement( valueElementName );
        valueElement.addText( String.valueOf( value ) );
        return Collections.singletonList( valueElement );
    }

    @Override
    public Object toNativeObject( )
    {
        return value;
    }

    public String toDebugString( final Locale locale )
    {
        final Locale loc = ( locale == null )
                ? PwmConstants.DEFAULT_LOCALE
                : locale;
        return value
                ? Display.getLocalizedMessage( loc, Display.Value_True, null )
                : Display.getLocalizedMessage( loc, Display.Value_False, null );
    }

    @Override
    public Serializable toDebugJsonObject( final Locale locale )
    {
        return value;
    }

    @Override
    public boolean requiresStoredUpdate( )
    {
        return false;
    }

    @Override
    public int currentSyntaxVersion( )
    {
        return 0;
    }

    @Override
    public String valueHash( ) throws PwmUnrecoverableException
    {
        return value ? "1" : "0";
    }
}
