/*
 * Password Management Servlets (PWM)
 * http://www.pwm-project.org
 *
 * Copyright (c) 2006-2009 Novell, Inc.
 * Copyright (c) 2009-2018 The PWM Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package password.pwm.svc.cache;

import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Value;
import password.pwm.bean.UserIdentity;

import java.io.Serializable;
import java.util.Objects;

@AllArgsConstructor( access = AccessLevel.PRIVATE )
@Value
public class CacheKey implements Serializable
{
    final Class srcClass;
    final UserIdentity userIdentity;
    final String valueID;

    public static CacheKey newKey(
            final Class srcClass,
            final UserIdentity userIdentity,
            final String valueID
    )
    {
        Objects.requireNonNull( srcClass, "srcClass can not be null" );
        Objects.requireNonNull( valueID, "valueID can not be null" );

        if ( valueID.isEmpty() )
        {
            throw new IllegalArgumentException( "valueID can not be empty" );
        }
        return new CacheKey( srcClass, userIdentity, valueID );
    }
}
